// File:    dvm3_rotmat.cc
// Author:  Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of dvm3
//
// dvm3 is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// dvm3 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <dvm3/dvm3_rotmat.h>   // dvm3_RotMat
#include <cstdlib>              // exit()
#include <iostream>

using namespace std;

/****************************************************************************
 */

//#########################################################################
// dtor, ctors, initializers...
//#########################################################################


//=========================================================================
dvm3_RotMat::
dvm3_RotMat()
  : dvm3_Matrix()
{
  vm_VMath<double,ENelts>::set( data_.m_, 0.0 );
  data_.m_[0] = 1.0;    data_.m_[4] = 1.0;    data_.m_[8] = 1.0;
}

//=========================================================================
dvm3_RotMat::
dvm3_RotMat( dvm3_Vector const& x,
             dvm3_Vector const& y,
             dvm3_Vector const& z,
             double       const  tol )
  : dvm3_Matrix(x, y, z)
{
  if (! is_rotation_matrix(tol) )
  {
    std::cerr << "dvm3_RotMat::dvm3_RotMat( dvm3_Vector const&,\n"
              << "                          dvm3_Vector const&,\n"
              << "                          dvm3_Vector const&)\n"
              << "   input vectors are not orthonormal to within tolerance "
              << tol << std::endl;
    std::exit(1);
  }
  else
  {
    orthonormalize();
  }
}

//=========================================================================
dvm3_RotMat::
dvm3_RotMat( double const x[],
             double const y[],
             double const z[],
             double const tol )
  : dvm3_Matrix(x, y, z)
{
  if (! is_rotation_matrix(tol) )
  {
    std::cerr << "dvm3_RotMat::dvm3_RotMat( double const v0[],\n" 
              << "                          double const v1[],\n"
              << "                          double const v2[])\n"
              << "   input vectors are not orthonormal to within tolerance " 
              << tol << std::endl;
    std::exit(1);
  }
  else
  {
    orthonormalize();
  }
}

//=========================================================================
//
// construct rotation matrix based on 3 Euler angles; the type of Euler
// angles is specified by angle_type.
//
// The matrix generated by this routine does the opposite
// of what the original rigid body motion did to the body
// centered coordinate system.  in other words, it does
// what is required to a location or direction to transform
// it from std to bcs.
//
// NOTE:  arguments are in radians.

dvm3_RotMat::
dvm3_RotMat( double angle0, double angle1, double angle2,
             dvm3_RotMat::EEulerAngleType angle_type )
{
  init( angle0, angle1, angle2, angle_type );
}

//=========================================================================
//
//: Construct a rotation matrix based on two (non-colinear) vectors.
//  The rotation matrix is constructed by sequential application
//  cross products.
//  <p>
//  If type == E_XY, the vectors passed in are assumed to lie in the
//  x-y plane with v0 an x-like vector and v1 a y-like vector;
//  e_x is v0.  e_z is constructed from e_x cross v1.
//  e_y is constructed from e_z cross e_x.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_YZ, the vectors passed in are assumed to lie in the
//  y-z plane with v0 an y-like vector and v1 a z-like vector;
//  e_y is v0.  e_x is constructed from e_y cross v1.
//  e_z is constructed from e_x cross e_y.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_ZX, the vectors passed in are assumed to lie in the
//  x-z plane with v0 an z-like vector and v1 a x-like vector;
//  e_z is v0.  e_y is constructed from e_z cross v1.
//  e_x is constructed from e_y cross e_z.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_YX, the vectors passed in are assumed to lie in the
//  x-y plane with v0 a y-like vector and v1 an x-like vector;
//  e_y is v0.  e_z is constructed from v1 cross e_y.
//  e_x is constructed from e_y cross e_z.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_ZY, the vectors passed in are assumed to lie in the
//  y-z plane with v0 a z-like vector and v1 a y-like vector;
//  e_z is v0.  e_x is constructed from v1 cross e_z.
//  e_y is constructed from e_z cross e_x.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_XZ, the vectors passed in are assumed to lie in the
//  x-z plane with v0 an x-like vector and v1 a z-like vector;
//  e_x is v0.  e_y is constructed from v1 cross e_x.
//  e_z is constructed from e_x cross e_y.  The three vectors 
//  e_x, e_y, and e_z are normalized.

dvm3_RotMat::
dvm3_RotMat( dvm3_Vector const& v0,
             dvm3_Vector const& v1,
             EInputVecPair      type
           )
{
  init( v0, v1, type );
}

//=========================================================================
//
//: Construct a rotation matrix based on two (non-colinear) vectors.
//  The rotation matrix is constructed by sequential application
//  cross products.
//  <p>
//  If type == E_XY, the vectors passed in are assumed to lie in the
//  x-y plane with v0 an x-like vector and v1 a y-like vector;
//  e_x is v0.  e_z is constructed from e_x cross v1.
//  e_y is constructed from e_z cross e_x.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_YZ, the vectors passed in are assumed to lie in the
//  y-z plane with v0 an y-like vector and v1 a z-like vector;
//  e_y is v0.  e_x is constructed from e_y cross v1.
//  e_z is constructed from e_x cross e_y.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_ZX, the vectors passed in are assumed to lie in the
//  x-z plane with v0 an z-like vector and v1 a x-like vector;
//  e_z is v0.  e_y is constructed from e_z cross v1.
//  e_x is constructed from e_y cross e_z.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_YX, the vectors passed in are assumed to lie in the
//  x-y plane with v0 a y-like vector and v1 an x-like vector;
//  e_y is v0.  e_z is constructed from v1 cross e_y.
//  e_x is constructed from e_y cross e_z.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_ZY, the vectors passed in are assumed to lie in the
//  y-z plane with v0 a z-like vector and v1 a y-like vector;
//  e_z is v0.  e_x is constructed from v1 cross e_z.
//  e_y is constructed from e_z cross e_x.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_XZ, the vectors passed in are assumed to lie in the
//  x-z plane with v0 an x-like vector and v1 a z-like vector;
//  e_x is v0.  e_y is constructed from v1 cross e_x.
//  e_z is constructed from e_x cross e_y.  The three vectors 
//  e_x, e_y, and e_z are normalized.

dvm3_RotMat::
dvm3_RotMat( double const  v0[],
             double const  v1[],
             EInputVecPair type
           )
{
  init( v0, v1, type );
}

//#########################################################################
// initializers...
//#########################################################################

//=========================================================================
//
// Initialize a rotation matrix based on 3 Euler angles; the type of Euler
// angles is specified by angle_type.
//
// The matrix generated by this routine does the opposite
// of what the original rigid body motion did to the body
// centered coordinate system.  in other words, it does
// what is required to a location or direction to transform
// it from std to bcs.
//
// NOTE:  arguments are in radians.

void dvm3_RotMat::
init( double angle0, double angle1, double angle2,
      dvm3_RotMat::EEulerAngleType angle_type )
{
  switch ( angle_type )
  {
  case EOSAC_az_el_clock:
    {
      double cos_az  =  cos(angle0);
      double sin_az  =  sin(angle0);
      double cos_el  =  cos(angle1);
      double sin_el  =  sin(angle1);
      double cos_z   =  cos(angle2);
      double sin_z   =  sin(angle2);

      vm_V3Math<double>::set( &data_.m_[ERow0],
                               cos_z * cos_az - sin_z * sin_el * sin_az,
                              -sin_z * cos_az - cos_z * sin_el * sin_az,
                                                        cos_el * sin_az );

      vm_V3Math<double>::set( &data_.m_[ERow1], sin_z * cos_el,
                                                cos_z * cos_el,
                                                        sin_el );

      vm_V3Math<double>::set( &data_.m_[ERow2],
                              -cos_z * sin_az - sin_z * sin_el * cos_az,
                               sin_z * sin_az - cos_z * sin_el * cos_az,
                                                        cos_el * cos_az );
    }
    break;
  case EX_convention:
    {
      double cos_phi   = cos( angle0 );
      double sin_phi   = sin( angle0 );
      double cos_theta = cos( angle1 );
      double sin_theta = sin( angle1 );
      double cos_psi   = cos( angle2 );
      double sin_psi   = sin( angle2 );

      vm_V3Math<double>::set( 
	&data_.m_[ERow0],
	cos_psi * cos_phi - cos_theta * sin_phi * sin_psi,
	cos_psi * sin_phi + cos_theta * cos_phi * sin_psi,
	sin_psi * sin_theta 
	);

      vm_V3Math<double>::set(
	&data_.m_[ERow1], 
	-1.0 * sin_psi * cos_phi - cos_theta * sin_phi * cos_psi,
	-1.0 * sin_psi * sin_phi + cos_theta * cos_phi * cos_psi,
	cos_psi * sin_theta
	);

      vm_V3Math<double>::set( 
	&data_.m_[ERow2],
	sin_theta * sin_phi,
	-1.0 * sin_theta * cos_phi,
	cos_theta
	);
    }
    break;
  case EY_convention:
    {
      double cos_phi   = cos( angle0 );
      double sin_phi   = sin( angle0 );
      double cos_theta = cos( angle1 );
      double sin_theta = sin( angle1 );
      double cos_psi   = cos( angle2 );
      double sin_psi   = sin( angle2 );

      vm_V3Math<double>::set( 
	&data_.m_[ERow0],
	-1.0 * sin_psi * sin_phi + cos_theta * cos_phi * cos_psi,
	       sin_psi * cos_phi + cos_theta * sin_phi * cos_psi,
	-1.0 * cos_psi * sin_theta 
	);

      vm_V3Math<double>::set(
	&data_.m_[ERow1], 
	-1.0 * cos_psi * sin_phi - cos_theta * cos_phi * sin_psi,
	       cos_psi * cos_phi - cos_theta * sin_phi * sin_psi,
	       sin_psi * sin_theta
	);

      vm_V3Math<double>::set( 
	&data_.m_[ERow2],
	sin_theta * cos_phi,
	sin_theta * sin_phi,
	cos_theta
	);
    }
    break;
  default:
    {
      std::cerr << "dvm3_RotMat::dvm3_RotMat( double angle0,\n"
                << "                          double angle1,\n"
                << "                          double angle2,\n"
                << "                          EEulerAngleType type )\n"
                << "   invalid Euler angle type encountered." << std::endl;
      std::exit(1);
    }
    break;
  }
}

//=========================================================================
//
//: Initialize a rotation matrix based on two (non-colinear) vectors.
//  The rotation matrix is constructed by sequential application
//  cross products.
//  <p>
//  If type == E_XY, the vectors passed in are assumed to lie in the
//  x-y plane with v0 an x-like vector and v1 a y-like vector;
//  e_x is v0.  e_z is constructed from e_x cross v1.
//  e_y is constructed from e_z cross e_x.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_YZ, the vectors passed in are assumed to lie in the
//  y-z plane with v0 an y-like vector and v1 a z-like vector;
//  e_y is v0.  e_x is constructed from e_y cross v1.
//  e_z is constructed from e_x cross e_y.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_ZX, the vectors passed in are assumed to lie in the
//  x-z plane with v0 an z-like vector and v1 a x-like vector;
//  e_z is v0.  e_y is constructed from e_z cross v1.
//  e_x is constructed from e_y cross e_z.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_YX, the vectors passed in are assumed to lie in the
//  x-y plane with v0 a y-like vector and v1 an x-like vector;
//  e_y is v0.  e_z is constructed from v1 cross e_y.
//  e_x is constructed from e_y cross e_z.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_ZY, the vectors passed in are assumed to lie in the
//  y-z plane with v0 a z-like vector and v1 a y-like vector;
//  e_z is v0.  e_x is constructed from v1 cross e_z.
//  e_y is constructed from e_z cross e_x.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_XZ, the vectors passed in are assumed to lie in the
//  x-z plane with v0 an x-like vector and v1 a z-like vector;
//  e_x is v0.  e_y is constructed from v1 cross e_x.
//  e_z is constructed from e_x cross e_y.  The three vectors 
//  e_x, e_y, and e_z are normalized.

void dvm3_RotMat::
init( dvm3_Vector const& v0,
      dvm3_Vector const& v1,
      EInputVecPair      type
    )
{
  switch ( type )
  {
  case E_XY:
     vm_V3Math<double>::copy(   &data_.m_[ERow0],  v0.data_.x_ );
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     vm_V3Math<double>::cross(  &data_.m_[ERow2], 
                                &data_.m_[ERow0],  v1.data_.x_);
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     vm_V3Math<double>::cross(  &data_.m_[ERow1], 
                                &data_.m_[ERow2],  &data_.m_[ERow0]);
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     break;

  case E_YZ:
     vm_V3Math<double>::copy(   &data_.m_[ERow1],  v0.data_.x_ );
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     vm_V3Math<double>::cross(  &data_.m_[ERow0],
                                &data_.m_[ERow1],  v1.data_.x_);
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     vm_V3Math<double>::cross(  &data_.m_[ERow2],
                                &data_.m_[ERow0],  &data_.m_[ERow1]);
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     break;

  case E_ZX:
     vm_V3Math<double>::copy(   &data_.m_[ERow2],  v0.data_.x_ );
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     vm_V3Math<double>::cross(  &data_.m_[ERow1],
                                &data_.m_[ERow2],  v1.data_.x_);
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     vm_V3Math<double>::cross(  &data_.m_[ERow0],
                                &data_.m_[ERow1],  &data_.m_[ERow2]);
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     break;

  case E_XZ:
     vm_V3Math<double>::copy(   &data_.m_[ERow0],  v0.data_.x_ );
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     vm_V3Math<double>::cross(  &data_.m_[ERow1],
                                v1.data_.x_,       &data_.m_[ERow0]);
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     vm_V3Math<double>::cross(  &data_.m_[ERow2],
                                &data_.m_[ERow0],  &data_.m_[ERow1]);
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     break;

  case E_ZY:
     vm_V3Math<double>::copy(   &data_.m_[ERow2],  v0.data_.x_ );
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     vm_V3Math<double>::cross(  &data_.m_[ERow0],
                                v1.data_.x_,       &data_.m_[ERow2]);
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     vm_V3Math<double>::cross(  &data_.m_[ERow1],
                                &data_.m_[ERow2],  &data_.m_[ERow0]);
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     break;

  case E_YX:
     vm_V3Math<double>::copy(   &data_.m_[ERow1],  v0.data_.x_ );
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     vm_V3Math<double>::cross(  &data_.m_[ERow2],
                                v1.data_.x_,       &data_.m_[ERow1]);
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     vm_V3Math<double>::cross(  &data_.m_[ERow0],
                                &data_.m_[ERow1],  &data_.m_[ERow2]);
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     break;
  }
}

//=========================================================================
//
//: Initialize a rotation matrix based on two (non-colinear) vectors.
//  The rotation matrix is constructed by sequential application
//  cross products.
//  <p>
//  If type == E_XY, the vectors passed in are assumed to lie in the
//  x-y plane with v0 an x-like vector and v1 a y-like vector;
//  e_x is v0.  e_z is constructed from e_x cross v1.
//  e_y is constructed from e_z cross e_x.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_YZ, the vectors passed in are assumed to lie in the
//  y-z plane with v0 an y-like vector and v1 a z-like vector;
//  e_y is v0.  e_x is constructed from e_y cross v1.
//  e_z is constructed from e_x cross e_y.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_ZX, the vectors passed in are assumed to lie in the
//  x-z plane with v0 an z-like vector and v1 a x-like vector;
//  e_z is v0.  e_y is constructed from e_z cross v1.
//  e_x is constructed from e_y cross e_z.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_YX, the vectors passed in are assumed to lie in the
//  x-y plane with v0 a y-like vector and v1 an x-like vector;
//  e_y is v0.  e_z is constructed from v1 cross e_y.
//  e_x is constructed from e_y cross e_z.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_ZY, the vectors passed in are assumed to lie in the
//  y-z plane with v0 a z-like vector and v1 a y-like vector;
//  e_z is v0.  e_x is constructed from v1 cross e_z.
//  e_y is constructed from e_z cross e_x.  The three vectors 
//  e_x, e_y, and e_z are normalized.
//  <p>
//  If type == E_XZ, the vectors passed in are assumed to lie in the
//  x-z plane with v0 an x-like vector and v1 a z-like vector;
//  e_x is v0.  e_y is constructed from v1 cross e_x.
//  e_z is constructed from e_x cross e_y.  The three vectors 
//  e_x, e_y, and e_z are normalized.

void dvm3_RotMat::
init( double const  v0[],
      double const  v1[],
      EInputVecPair type
    )
{
  switch ( type )
  {
  case E_XY:
     vm_V3Math<double>::copy(   &data_.m_[ERow0],  v0 );
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     vm_V3Math<double>::cross(  &data_.m_[ERow2],
                                &data_.m_[ERow0],  v1 );
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     vm_V3Math<double>::cross(  &data_.m_[ERow1],
                                &data_.m_[ERow2],  &data_.m_[ERow0]);
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     break;

  case E_YZ:
     vm_V3Math<double>::copy(   &data_.m_[ERow1],  v0 );
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     vm_V3Math<double>::cross(  &data_.m_[ERow0],
                                &data_.m_[ERow1],  v1 );
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     vm_V3Math<double>::cross(  &data_.m_[ERow2],
                                &data_.m_[ERow0],  &data_.m_[ERow1]);
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     break;

  case E_ZX:
     vm_V3Math<double>::copy(   &data_.m_[ERow2],  v0 );
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     vm_V3Math<double>::cross(  &data_.m_[ERow1],
                                &data_.m_[ERow2],  v1 );
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     vm_V3Math<double>::cross(  &data_.m_[ERow0],
                                &data_.m_[ERow1],  &data_.m_[ERow2]);
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     break;

  case E_XZ:
     vm_V3Math<double>::copy(   &data_.m_[ERow0],  v0 );
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     vm_V3Math<double>::cross(  &data_.m_[ERow1],
                                v1,                &data_.m_[ERow0]);
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     vm_V3Math<double>::cross(  &data_.m_[ERow2],
                                &data_.m_[ERow0],  &data_.m_[ERow1]);
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     break;

  case E_ZY:
     vm_V3Math<double>::copy(   &data_.m_[ERow2],  v0 );
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     vm_V3Math<double>::cross(  &data_.m_[ERow0],
                                v1,                &data_.m_[ERow2]);
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     vm_V3Math<double>::cross(  &data_.m_[ERow1],
                                &data_.m_[ERow2],  &data_.m_[ERow0]);
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     break;

  case E_YX:
     vm_V3Math<double>::copy(   &data_.m_[ERow1],  v0 );
     vm_V3Math<double>::unitize(&data_.m_[ERow1]);
     vm_V3Math<double>::cross(  &data_.m_[ERow2],
                                v1,                &data_.m_[ERow1]);
     vm_V3Math<double>::unitize(&data_.m_[ERow2]);
     vm_V3Math<double>::cross(  &data_.m_[ERow0],
                                &data_.m_[ERow1],  &data_.m_[ERow2]);
     vm_V3Math<double>::unitize(&data_.m_[ERow0]);
     break;
  }
}

//=========================================================================
// mutators

void dvm3_RotMat::
invert()
{
  #ifdef _MYSWAP_
  #error "dvm3_RotMat::invert() - _MYSWAP_ already defined!"
  #else
  #define _MYSWAP_(a,b,temp) {temp=a; a=b; b=temp;}
  #endif
  double tmp;
  _MYSWAP_( data_.m_[1], data_.m_[3], tmp )
  _MYSWAP_( data_.m_[2], data_.m_[6], tmp )
  _MYSWAP_( data_.m_[5], data_.m_[7], tmp )
}

//=========================================================================
// boolean functions

int dvm3_RotMat::
is_rotation_matrix( double const tol ) const
{
  int rc = 0;
  rc += !vm_V3Math<double>::is_unit_vector(&data_.m_[ERow0], tol);
  rc += !vm_V3Math<double>::is_unit_vector(&data_.m_[ERow1], tol);
  rc += !vm_V3Math<double>::is_unit_vector(&data_.m_[ERow2], tol);
  rc += !vm_V3Math<double>::are_orthogonal(&data_.m_[ERow0], 
                                           &data_.m_[ERow1], tol);
  rc += !vm_V3Math<double>::are_orthogonal(&data_.m_[ERow1], 
                                           &data_.m_[ERow2], tol);
  rc += !vm_V3Math<double>::are_orthogonal(&data_.m_[ERow2], 
                                           &data_.m_[ERow0], tol);
  return !rc;
}

//=========================================================================
// std::ostream&
// operator<<( std::ostream& os, const dvm3_RotMat& m )
// {
//   return m.print_on( os, "[", "]" );
// }
