// File:    t_dvm3_rotmat.cc
// Author:  Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of dvm3.cvs
//
// dvm3.cvs is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// dvm3.cvs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <dvm3/dvm3_rotmat.h>

//#include <mathconst.h>
#include <cstdlib>
#include <iostream>
#include <testlib/testlib.h>

// debug
#include <cfloat>

using namespace std;

static
int dvm3_RotMat_test01()    /* test default constructor */
{
  dvm3_RotMat m1;
  return 
    (m1[0][0] == 1.0) && (m1[0][1] == 0.0) && (m1[0][2] == 0.0) &&
    (m1[1][0] == 0.0) && (m1[1][1] == 1.0) && (m1[1][2] == 0.0) &&
    (m1[2][0] == 0.0) && (m1[2][1] == 0.0) && (m1[2][2] == 1.0);
}

static
int dvm3_RotMat_test02()    /* test construct from 3 dvm3_Vector's*/
{
  // note:  test matrix frmo equation 4-46 of Goldstein, "Classical Mechanics",
  //        2nd edition.
  double theta = 0.1;
  double phi   = 0.3;
  double psi   = 0.2;
  dvm3_Vector v0(  cos(psi)*cos(phi) - cos(theta)*sin(psi)*sin(phi),
                   cos(psi)*sin(phi) + cos(theta)*sin(psi)*cos(phi),
                                       sin(theta)*sin(psi)
                 );
  dvm3_Vector v1( -sin(psi)*cos(phi) - cos(theta)*cos(psi)*sin(phi),
                  -sin(psi)*sin(phi) + cos(theta)*cos(psi)*cos(phi),
                                       sin(theta)*cos(psi)
                 );
  dvm3_Vector v2(                      sin(theta)*sin(phi),
                                      -sin(theta)*cos(phi),
                                       cos(theta)
                 );
  dvm3_RotMat m1( v0, v1, v2 );
  return
    m1.is_rotation_matrix();
}

static
int dvm3_RotMat_test03()    /* test construct from 3 double[]'s*/
{
  double theta = 0.2;
  double phi   = 0.3;
  double psi   = 0.4;
  double v0[] = {  cos(psi)*cos(phi) - cos(theta)*sin(psi)*sin(phi),
                   cos(psi)*sin(phi) + cos(theta)*sin(psi)*cos(phi),
                                       sin(theta)*sin(psi)
                };
  double v1[] = { -sin(psi)*cos(phi) - cos(theta)*cos(psi)*sin(phi),
                  -sin(psi)*sin(phi) + cos(theta)*cos(psi)*cos(phi),
                                       sin(theta)*cos(psi)
                };
  double v2[] = {                      sin(theta)*sin(phi),
                                      -sin(theta)*cos(phi),
                                       cos(theta)
                };
  dvm3_RotMat m1( v0, v1, v2 );
  return
    m1.is_rotation_matrix();
}

static
int dvm3_RotMat_test04()    /* test copy constructor */
{
  dvm3_RotMat m1;
  return 
    (m1[0][0] == 1.0) && (m1[0][1] == 0.0) && (m1[0][2] == 0.0) &&
    (m1[1][0] == 0.0) && (m1[1][1] == 1.0) && (m1[1][2] == 0.0) &&
    (m1[2][0] == 0.0) && (m1[2][1] == 0.0) && (m1[2][2] == 1.0);
}

static
int dvm3_RotMat_test05()    /* test pure z rotation */
{
  double az   = 0.0;
  double el   = 0.0;
  double zrot = 0.1;
  dvm3_RotMat m(az, el, zrot);

  return
    !fuzzy_compare(m[0][0],
                   9.9500416527802582056239089070004e-1 ) &&
    !fuzzy_compare(m[0][1],
                  -9.9833416646828154750181738563697e-2 ) &&
    !fuzzy_compare(m[0][2],
                   0.0       ) &&

    !fuzzy_compare(m[1][0],
                   9.9833416646828154750181738563697e-2 ) &&
    !fuzzy_compare(m[1][1],
                   9.9500416527802582056239089070004e-1 ) &&
    !fuzzy_compare(m[1][2],
                   0.0                                  ) &&

    !fuzzy_compare(m[2][0],
                   0.0       ) &&
    !fuzzy_compare(m[2][1],
                   0.0       ) &&
    !fuzzy_compare(m[2][2],
                   1.0       );
  /*
  return

    (m[0][0] ==  cos(-zrot)) && 
    (m[0][1] ==  sin(-zrot)) && 
    (m[0][2] ==  0.0       ) &&

    (m[1][0] == -sin(-zrot)) && 
    (m[1][1] ==  cos(-zrot)) && 
    (m[1][2] ==  0.0       ) &&

    (m[2][0] ==  0.0       ) && 
    (m[2][1] ==  0.0       ) && 
    (m[2][2] ==  1.0       );
    */

}

static
int dvm3_RotMat_test05a()    /* test x-convention rotation angles */
{
  dvm3_Vector pos(1.0,2.0,3.0);
  dvm3_Vector res;

  //Rot(phi,theta,psi) = Rot(Z,psi)Rot(X,theta)Rot(Z,phi);
  //                   = Applied as
  //                   =  three  <- two      <- one

  double phi   =        0.0       ;
  double theta =        M_PI / 2.0;
  double psi   = -1.0 * M_PI / 2.0;

  dvm3_RotMat m(phi, theta, psi, dvm3_RotMat::EX_convention);
  m.mvmult( res, pos );

 return
    !fuzzy_compare(m[0][0],  0.0) &&
    !fuzzy_compare(m[0][1],  0.0) &&
    !fuzzy_compare(m[0][2], -1.0) &&

    !fuzzy_compare(m[1][0],  1.0) &&
    !fuzzy_compare(m[1][1],  0.0) &&
    !fuzzy_compare(m[1][2],  0.0) &&

    !fuzzy_compare(m[2][0],  0.0) &&
    !fuzzy_compare(m[2][1], -1.0) &&
    !fuzzy_compare(m[2][2],  0.0) &&

    !fuzzy_compare(res[0],  -3.0) &&
    !fuzzy_compare(res[1],   1.0) &&
    !fuzzy_compare(res[2],  -2.0) ;


  /*
  return
    !fuzzy_compare(m[0][0],  
		   cos(psi)*cos(phi)-cos(theta)*sin(phi)*sin(psi)) && 
    !fuzzy_compare(m[0][1],  
		   cos(psi)*sin(phi)+cos(theta)*cos(phi)*sin(psi)) && 
    !fuzzy_compare(m[0][2],  
		   sin(psi)*sin(theta)) &&
       
    !fuzzy_compare(m[1][0],  
		   -1.0*sin(psi)*cos(phi)-cos(theta)*sin(phi)*cos(psi)) && 
    !fuzzy_compare(m[1][1],  
		   -1.0*sin(psi)*sin(phi)+cos(theta)*cos(phi)*cos(psi)) && 
    !fuzzy_compare(m[1][2],  
		   cos(psi)*sin(theta)) &&
       
    !fuzzy_compare(m[2][0],  
		   sin(theta)*sin(phi)) && 
    !fuzzy_compare(m[2][1],  
		   -1.0*sin(theta)*cos(phi)) && 
    !fuzzy_compare(m[2][2],  
		   cos(theta));
  */
}

static
int dvm3_RotMat_test05b()    /* test y-convention rotation angles */
{
  dvm3_Vector pos(1.0,2.0,3.0);
  dvm3_Vector res;

  //Rot(phi,theta,psi) = Rot(Z,psi)Rot(Y,theta)Rot(Z,phi);
  //                   = Applied as
  //                   =  three  <- two      <- one

  double phi   = -1.0 * M_PI / 2.0;
  double theta =        M_PI / 2.0;
  double psi   =        0.0       ;

  dvm3_RotMat m(phi, theta, psi, dvm3_RotMat::EY_convention);
  m.mvmult( res, pos );

  return
    !fuzzy_compare(m[0][0],  0.0) &&
    !fuzzy_compare(m[0][1],  0.0) &&
    !fuzzy_compare(m[0][2], -1.0) &&

    !fuzzy_compare(m[1][0],  1.0) &&
    !fuzzy_compare(m[1][1],  0.0) &&
    !fuzzy_compare(m[1][2],  0.0) &&

    !fuzzy_compare(m[2][0],  0.0) &&
    !fuzzy_compare(m[2][1], -1.0) &&
    !fuzzy_compare(m[2][2],  0.0) &&

    !fuzzy_compare(res[0],  -3.0) &&
    !fuzzy_compare(res[1],   1.0) &&
    !fuzzy_compare(res[2],  -2.0) ;

  /*
    !fuzzy_compare(m[0][0],  
		   -1.0 * sin(psi)*sin(phi)+cos(theta)*cos(phi)*cos(psi)) && 
    !fuzzy_compare(m[0][1],  
		          sin(psi)*cos(phi)+cos(theta)*sin(phi)*cos(psi)) && 
    !fuzzy_compare(m[0][2],
		   -1.0 * cos(psi)*sin(theta)) &&
     
    !fuzzy_compare(m[1][0],
		   -1.0 * cos(psi)*sin(phi)-cos(theta)*cos(phi)*sin(psi)) && 
    !fuzzy_compare(m[1][1],
		   cos(psi)*cos(phi)-cos(theta)*sin(phi)*sin(psi) ) && 
    !fuzzy_compare(m[1][2],
		          sin(psi)*sin(theta)) &&
    
    !fuzzy_compare(m[2][0],
		   sin(theta)*cos(phi) ) && 
    !fuzzy_compare(m[2][1],
		   sin(theta)*sin(phi) ) && 
    !fuzzy_compare(m[2][2],
		   cos(theta) );
  */
}

static
int dvm3_RotMat_test06()    /* test pure elevation */
{
  double az   = 0.0;
  double el   = 0.1;
  double zrot = 0.0;
  dvm3_RotMat m(az, el, zrot);

  return
    !fuzzy_compare(m[0][0],  1.0                                 ) &&
    !fuzzy_compare(m[0][1],  0.0                                 ) &&
    !fuzzy_compare(m[0][2],  0.0                                 ) &&

    !fuzzy_compare(m[1][0],  0.0                                 ) &&
    !fuzzy_compare(m[1][1],  9.9500416527802582056239089070004e-1) &&
    !fuzzy_compare(m[1][2],  9.9833416646828154750181738563697e-2) &&

    !fuzzy_compare(m[2][0],  0.0                                 ) &&
    !fuzzy_compare(m[2][1], -9.9833416646828154750181738563697e-2) &&
    !fuzzy_compare(m[2][2],  9.9500416527802582056239089070004e-1) ;

  /*
  return
    !fuzzy_compare(m[0][0],  1.0       ) &&
    !fuzzy_compare(m[0][1],  0.0       ) &&
    !fuzzy_compare(m[0][2],  0.0       ) &&

    !fuzzy_compare(m[1][0],  0.0       ) &&
    !fuzzy_compare(m[1][1],  cos(-el)  ) &&
    !fuzzy_compare(m[1][2], -sin(-el)  ) &&
       
    !fuzzy_compare(m[2][0],  0.0       ) &&
    !fuzzy_compare(m[2][1],  sin(-el)  ) &&
    !fuzzy_compare(m[2][2],  cos(-el)  );
  */
}

static
int dvm3_RotMat_test07()    /* test pure azimuth */
{
  double az   = 0.1;
  double el   = 0.0;
  double zrot = 0.0;
  dvm3_RotMat m(az, el, zrot);

  return
    !fuzzy_compare(m[0][0],  9.9500416527802582056239089070004e-1) &&
    !fuzzy_compare(m[0][1],  0.0                                 ) &&
    !fuzzy_compare(m[0][2],  9.9833416646828154750181738563697e-2) &&

    !fuzzy_compare(m[1][0],  0.0                                 ) &&
    !fuzzy_compare(m[1][1],  1.0                                 ) &&
    !fuzzy_compare(m[1][2],  0.0                                 ) &&

    !fuzzy_compare(m[2][0], -9.9833416646828154750181738563697e-2) &&
    !fuzzy_compare(m[2][1],  0.0                                 ) &&
    !fuzzy_compare(m[2][2],  9.9500416527802582056239089070004e-1) ;

  /*
  return
    !fuzzy_compare(m[0][0],  cos(-az)  ) &&
    !fuzzy_compare(m[0][1],  0.0       ) &&
    !fuzzy_compare(m[0][2], -sin(-az)  ) &&

    !fuzzy_compare(m[1][0],  0.0       ) &&
    !fuzzy_compare(m[1][1],  1.0       ) &&
    !fuzzy_compare(m[1][2],  0.0       ) &&

    !fuzzy_compare(m[2][0],  sin(-az)  ) &&
    !fuzzy_compare(m[2][1],  0.0       ) &&
    !fuzzy_compare(m[2][2],  cos(-az)  );
    */
}

static
int dvm3_RotMat_test08()    /* test azimuth, elevation, z-clocking */
{
  double az   = M_PI_2;
  double el   = M_PI_2;
  double zrot = M_PI_2;
  dvm3_RotMat m(az, el, zrot);

  return !fuzzy_compare(m[0][0], -1.0) &&
         !fuzzy_compare(m[0][1],  0.0) &&
         !fuzzy_compare(m[0][2],  0.0) &&
         !fuzzy_compare(m[1][0],  0.0) &&
         !fuzzy_compare(m[1][1],  0.0) &&
         !fuzzy_compare(m[1][2],  1.0) &&
         !fuzzy_compare(m[2][0],  0.0) &&
         !fuzzy_compare(m[2][1],  1.0) &&
         !fuzzy_compare(m[2][2],  0.0);
}

static
int dvm3_RotMat_test09()    /* test 2 vectors ctor, type XY */
{
  int rc0, rc1;
  {
    dvm3_Vector const v0( 0.0, 1.0, 0.0 );
    dvm3_Vector const v1( 0.0, 0.0, 1.0 );
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_XY );

    rc0 =  !fuzzy_compare(m[0][0],  0.0) &&
           !fuzzy_compare(m[0][1],  1.0) &&
           !fuzzy_compare(m[0][2],  0.0) &&
           !fuzzy_compare(m[1][0],  0.0) &&
           !fuzzy_compare(m[1][1],  0.0) &&
           !fuzzy_compare(m[1][2],  1.0) &&
           !fuzzy_compare(m[2][0],  1.0) &&
           !fuzzy_compare(m[2][1],  0.0) &&
           !fuzzy_compare(m[2][2],  0.0);
  }
  {
    double const v0[] = { 0.0, 1.0, 0.0 };
    double const v1[] = { 0.0, 0.0, 1.0 };
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_XY );

    rc1 =  !fuzzy_compare(m[0][0],  0.0) &&
           !fuzzy_compare(m[0][1],  1.0) &&
           !fuzzy_compare(m[0][2],  0.0) &&
           !fuzzy_compare(m[1][0],  0.0) &&
           !fuzzy_compare(m[1][1],  0.0) &&
           !fuzzy_compare(m[1][2],  1.0) &&
           !fuzzy_compare(m[2][0],  1.0) &&
           !fuzzy_compare(m[2][1],  0.0) &&
           !fuzzy_compare(m[2][2],  0.0);
  }
  return rc0 && rc1;
}

static
int dvm3_RotMat_test10()    /* test 2 vectors ctor, type YZ */
{
  int rc0, rc1;
  {
    dvm3_Vector const v0( 0.0, 1.0, 0.0 );
    dvm3_Vector const v1( 0.0, 0.0, 1.0 );
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_YZ );

    rc0 =  !fuzzy_compare(m[0][0],  1.0) &&
           !fuzzy_compare(m[0][1],  0.0) &&
           !fuzzy_compare(m[0][2],  0.0) &&
           !fuzzy_compare(m[1][0],  0.0) &&
           !fuzzy_compare(m[1][1],  1.0) &&
           !fuzzy_compare(m[1][2],  0.0) &&
           !fuzzy_compare(m[2][0],  0.0) &&
           !fuzzy_compare(m[2][1],  0.0) &&
           !fuzzy_compare(m[2][2],  1.0);
  }
  {
    double const v0[] = { 0.0, 1.0, 0.0 };
    double const v1[] = { 0.0, 0.0, 1.0 };
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_YZ );

    rc1 =  !fuzzy_compare(m[0][0],  1.0) &&
           !fuzzy_compare(m[0][1],  0.0) &&
           !fuzzy_compare(m[0][2],  0.0) &&
           !fuzzy_compare(m[1][0],  0.0) &&
           !fuzzy_compare(m[1][1],  1.0) &&
           !fuzzy_compare(m[1][2],  0.0) &&
           !fuzzy_compare(m[2][0],  0.0) &&
           !fuzzy_compare(m[2][1],  0.0) &&
           !fuzzy_compare(m[2][2],  1.0);
  }
  return rc0 && rc1;
}

static
int dvm3_RotMat_test11()    /* test 2 vectors ctor, type ZX */
{
  int rc0, rc1;
  {
    dvm3_Vector const v0( 0.0, 1.0, 0.0 );
    dvm3_Vector const v1( 0.0, 0.0, 1.0 );
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_ZX );

    rc0 =  !fuzzy_compare(m[0][0],  0.0) &&
           !fuzzy_compare(m[0][1],  0.0) &&
           !fuzzy_compare(m[0][2],  1.0) &&
           !fuzzy_compare(m[1][0],  1.0) &&
           !fuzzy_compare(m[1][1],  0.0) &&
           !fuzzy_compare(m[1][2],  0.0) &&
           !fuzzy_compare(m[2][0],  0.0) &&
           !fuzzy_compare(m[2][1],  1.0) &&
           !fuzzy_compare(m[2][2],  0.0);
  }
  {
    double const v0[] = { 0.0, 1.0, 0.0 };
    double const v1[] = { 0.0, 0.0, 1.0 };
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_ZX );

    rc1 =  !fuzzy_compare(m[0][0],  0.0) &&
           !fuzzy_compare(m[0][1],  0.0) &&
           !fuzzy_compare(m[0][2],  1.0) &&
           !fuzzy_compare(m[1][0],  1.0) &&
           !fuzzy_compare(m[1][1],  0.0) &&
           !fuzzy_compare(m[1][2],  0.0) &&
           !fuzzy_compare(m[2][0],  0.0) &&
           !fuzzy_compare(m[2][1],  1.0) &&
           !fuzzy_compare(m[2][2],  0.0);
  }
  return rc0 && rc1;
}

static
int dvm3_RotMat_test12()    /* test 2 vectors ctor, type XZ */
{
  int rc0, rc1;
  {
    dvm3_Vector const v0( 0.0, 1.0, 0.0 );
    dvm3_Vector const v1( 0.0, 0.0, 1.0 );
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_XZ );

    rc0 =  !fuzzy_compare(m[0][0],  0.0) &&
           !fuzzy_compare(m[0][1],  1.0) &&
           !fuzzy_compare(m[0][2],  0.0) &&
           !fuzzy_compare(m[1][0], -1.0) &&
           !fuzzy_compare(m[1][1],  0.0) &&
           !fuzzy_compare(m[1][2],  0.0) &&
           !fuzzy_compare(m[2][0],  0.0) &&
           !fuzzy_compare(m[2][1],  0.0) &&
           !fuzzy_compare(m[2][2],  1.0);
  }
  {
    double const v0[] = { 0.0, 1.0, 0.0 };
    double const v1[] = { 0.0, 0.0, 1.0 };
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_XZ );

    rc1 =  !fuzzy_compare(m[0][0],  0.0) &&
           !fuzzy_compare(m[0][1],  1.0) &&
           !fuzzy_compare(m[0][2],  0.0) &&
           !fuzzy_compare(m[1][0], -1.0) &&
           !fuzzy_compare(m[1][1],  0.0) &&
           !fuzzy_compare(m[1][2],  0.0) &&
           !fuzzy_compare(m[2][0],  0.0) &&
           !fuzzy_compare(m[2][1],  0.0) &&
           !fuzzy_compare(m[2][2],  1.0);
  }
  return rc0 && rc1;
}

static
int dvm3_RotMat_test13()    /* test 2 vectors ctor, type ZY */
{
  int rc0, rc1;
  {
    dvm3_Vector const v0( 0.0, 1.0, 0.0 );
    dvm3_Vector const v1( 0.0, 0.0, 1.0 );
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_ZY );

    rc0 =  !fuzzy_compare(m[0][0], -1.0) &&
           !fuzzy_compare(m[0][1],  0.0) &&
           !fuzzy_compare(m[0][2],  0.0) &&
           !fuzzy_compare(m[1][0],  0.0) &&
           !fuzzy_compare(m[1][1],  0.0) &&
           !fuzzy_compare(m[1][2],  1.0) &&
           !fuzzy_compare(m[2][0],  0.0) &&
           !fuzzy_compare(m[2][1],  1.0) &&
           !fuzzy_compare(m[2][2],  0.0);
  }
  {
    double const v0[] = { 0.0, 1.0, 0.0 };
    double const v1[] = { 0.0, 0.0, 1.0 };
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_ZY );

    rc1 =  !fuzzy_compare(m[0][0], -1.0) &&
           !fuzzy_compare(m[0][1],  0.0) &&
           !fuzzy_compare(m[0][2],  0.0) &&
           !fuzzy_compare(m[1][0],  0.0) &&
           !fuzzy_compare(m[1][1],  0.0) &&
           !fuzzy_compare(m[1][2],  1.0) &&
           !fuzzy_compare(m[2][0],  0.0) &&
           !fuzzy_compare(m[2][1],  1.0) &&
           !fuzzy_compare(m[2][2],  0.0);
  }
  return rc0 && rc1;
}

static
int dvm3_RotMat_test14()    /* test 2 vectors ctor, type YX */
{
  int rc0, rc1;
  {
    dvm3_Vector const v0( 0.0, 1.0, 0.0 );
    dvm3_Vector const v1( 0.0, 0.0, 1.0 );
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_YX );

    rc0 =  !fuzzy_compare(m[0][0],  0.0) &&
           !fuzzy_compare(m[0][1],  0.0) &&
           !fuzzy_compare(m[0][2],  1.0) &&
           !fuzzy_compare(m[1][0],  0.0) &&
           !fuzzy_compare(m[1][1],  1.0) &&
           !fuzzy_compare(m[1][2],  0.0) &&
           !fuzzy_compare(m[2][0], -1.0) &&
           !fuzzy_compare(m[2][1],  0.0) &&
           !fuzzy_compare(m[2][2],  0.0);
  }
  {
    double const v0[] = { 0.0, 1.0, 0.0 };
    double const v1[] = { 0.0, 0.0, 1.0 };
    dvm3_RotMat m(v0, v1, dvm3_RotMat::E_YX );

    rc1 =  !fuzzy_compare(m[0][0],  0.0) &&
           !fuzzy_compare(m[0][1],  0.0) &&
           !fuzzy_compare(m[0][2],  1.0) &&
           !fuzzy_compare(m[1][0],  0.0) &&
           !fuzzy_compare(m[1][1],  1.0) &&
           !fuzzy_compare(m[1][2],  0.0) &&
           !fuzzy_compare(m[2][0], -1.0) &&
           !fuzzy_compare(m[2][1],  0.0) &&
           !fuzzy_compare(m[2][2],  0.0);
  }
  return rc0 && rc1;
}

#if 0
static
int dvm3_RotMat_test09()    /* test small angle rotations */
{
  int rc0 = 0;
  {
    double rot_x = 1.e-9;
    double rot_y = 0.0;
    double rot_z = 0.0;
 
    dvm3_RotMat m; m.make_small_angle_rotation(rot_x, rot_y, rot_z);

    rc0 = !fuzzy_compare(m[0][0],  1.0  ) &&
          !fuzzy_compare(m[0][1],  0.0  ) &&
          !fuzzy_compare(m[0][2],  0.0  ) &&
          !fuzzy_compare(m[1][0],  0.0  ) &&
          !fuzzy_compare(m[1][1],  1.0  ) &&
          !fuzzy_compare(m[1][2], -rot_x) &&
          !fuzzy_compare(m[2][0],  0.0  ) &&
          !fuzzy_compare(m[2][1],  rot_x) &&
          !fuzzy_compare(m[2][2],  1.0  );
    cerr << "___0_____ " << rc0 << endl;
    m.print_on( cerr );
  }

  int rc1 = 0;
  {
    double rot_x = 0.0;
    double rot_y = 1.e-9;
    double rot_z = 0.0;

    dvm3_RotMat m; m.make_small_angle_rotation(rot_x, rot_y, rot_z);

    rc1 = !fuzzy_compare(m[0][0],  1.0  ) &&
          !fuzzy_compare(m[0][1],  0.0  ) &&
          !fuzzy_compare(m[0][2], -rot_y) &&
          !fuzzy_compare(m[1][0],  0.0  ) &&
          !fuzzy_compare(m[1][1],  1.0  ) &&
          !fuzzy_compare(m[1][2],  0.0  ) &&
          !fuzzy_compare(m[2][0],  rot_y) &&
          !fuzzy_compare(m[2][1],  0.0  ) &&
          !fuzzy_compare(m[2][2],  1.0  );
    cerr << "___1_____ " << rc1 << endl;
    m.print_on( cerr );
  } 
  int rc2 = 0;
  {
    double rot_x = 0.0;
    double rot_y = 0.0;
    double rot_z = 1.e-9;

    dvm3_RotMat m; m.make_small_angle_rotation(rot_x, rot_y, rot_z);

    rc2 = !fuzzy_compare(m[0][0],  1.0  ) &&
          !fuzzy_compare(m[0][1],  rot_z) &&
          !fuzzy_compare(m[0][2],  0.0  ) &&
          !fuzzy_compare(m[1][0], -rot_z) &&
          !fuzzy_compare(m[1][1],  1.0  ) &&
          !fuzzy_compare(m[1][2],  0.0  ) &&
          !fuzzy_compare(m[2][0],  0.0  ) &&
          !fuzzy_compare(m[2][1],  0.0  ) &&
          !fuzzy_compare(m[2][2],  1.0  );
    cerr << "___2_____ " << rc2 << endl;
    m.print_on( cerr );
  }
  
  // return rc0 && rc1 && rc2;
  return rc1;
}
#endif

int
main( int argc, char** argv )
{
  int ch;
  int printit = 0;
  while ( --argc > 0 && (*++argv)[0] == '-' )
  {
    while ( (ch = *++argv[0]) )
    {
      switch( ch )
      {
      case 'p':                   // print out diagnostics as we go along
        printit = 1;
        break;
      default:
        cerr << "Usage: " << argv[0] << " [-p]\n";
        return EXIT_FAILURE;
      }
    }
  }

  START("dvm3 package:  dvm3_RotMat");

  TEST( "default constructor",              dvm3_RotMat_test01(), 1 );

  TEST( "construct from dvm3_Vector's",     dvm3_RotMat_test02(), 1 );
  TEST( "construct from double[]'s",        dvm3_RotMat_test03(), 1 );

  TEST( "copy constructor",                 dvm3_RotMat_test04(), 1 );

  TEST( "pure z rotation",                  dvm3_RotMat_test05(), 1 );
  TEST( "x-convention rotation",            dvm3_RotMat_test05a(),1 );
  TEST( "y-convention rotation",            dvm3_RotMat_test05b(),1 );

  TEST( "pure el rotation",                 dvm3_RotMat_test06(), 1 );
  TEST( "pure az rotation",                 dvm3_RotMat_test07(), 1 );
  TEST( "azimuth, elevation, z-clocking",   dvm3_RotMat_test08(), 1 );
  TEST( "2 vectors ctor, type XY",          dvm3_RotMat_test09(), 1 );
  TEST( "2 vectors ctor, type YZ",          dvm3_RotMat_test10(), 1 );
  TEST( "2 vectors ctor, type ZX",          dvm3_RotMat_test11(), 1 );
  TEST( "2 vectors ctor, type XZ",          dvm3_RotMat_test12(), 1 );
  TEST( "2 vectors ctor, type ZY",          dvm3_RotMat_test13(), 1 );
  TEST( "2 vectors ctor, type YX",          dvm3_RotMat_test14(), 1 );
  #if 0
  TEST( "small angle rotations",            dvm3_RotMat_test09(), 1 );
  #endif

  SUMMARY();

  return FAILED() ? 1 : 0;
}

